@@ -11,26 +11,28 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kstdaction.h>
+#include <ktempfile.h>
 #include <ktip.h>
 #include <qtimer.h>
 
 #include <kparts/part.h>
 #include <kparts/genericfactory.h>
 
-#include "../config.h"
-#include "../kviewshell/marklist.h"
-#include "documentPagePixmap.h"
-#include "documentWidget.h"
-#include "fontpool.h"
 #include "kdvi_multipage.h"
-#include "kviewpart.h"
-#include "performanceMeasurement.h"
-#include "prefs.h"
-#include "kprinterwrapper.h"
+#include "documentWidget.h"
+#include "dviFile.h"
+#include "dviPageCache.h"
 #include "dviWidget.h"
-
+#include "fontpool.h"
+#include "kprinterwrapper.h"
+#include "kviewpart.h"
+#include "marklist.h"
 #include "optionDialogFontsWidget.h"
 #include "optionDialogSpecialWidget.h"
+#include "performanceMeasurement.h"
+#include "prefs.h"
+#include "renderedDocumentPagePixmap.h"
+
 
 #include <qlabel.h>
 
@@ -45,32 +47,31 @@ int  performanceFlag = 0;
 typedef KParts::GenericFactory<KDVIMultiPage> KDVIMultiPageFactory;
 K_EXPORT_COMPONENT_FACTORY(kdvipart, KDVIMultiPageFactory)
 
+
+
 KDVIMultiPage::KDVIMultiPage(QWidget *parentWidget, const char *widgetName, QObject *parent,
                              const char *name, const QStringList& args)
   : KMultiPage(parentWidget, widgetName, parent, name), DVIRenderer(parentWidget)
 {
+  Q_UNUSED(args);
 #ifdef PERFORMANCE_MEASUREMENT
   performanceTimer.start();
 #endif
 
-  timer_id = -1;
-  setInstance(KDVIMultiPageFactory::instance());
+  searchUsed = false;
 
-  printer = 0;
+  setInstance(KDVIMultiPageFactory::instance());
 
   // Points to the same object as renderer to avoid downcasting.
   // FIXME: Remove when the API of the Renderer-class is finished.
   DVIRenderer.setName("DVI renderer");
   setRenderer(&DVIRenderer);
 
-  docInfoAction    = new KAction(i18n("Document &Info"), 0, &DVIRenderer, SLOT(showInfo()), actionCollection(), "info_dvi");
-
-  embedPSAction      = new KAction(i18n("Embed External PostScript Files..."), 0, this, SLOT(slotEmbedPostScript()), actionCollection(), "embed_postscript");
-
+  docInfoAction    = new KAction(i18n("Document &Info"), "info", 0, &DVIRenderer, SLOT(showInfo()), actionCollection(), "info_dvi");
+  embedPSAction    = new KAction(i18n("Embed External PostScript Files..."), 0, this, SLOT(slotEmbedPostScript()), actionCollection(), "embed_postscript");
   new KAction(i18n("Enable All Warnings && Messages"), 0, this, SLOT(doEnableWarnings()), actionCollection(), "enable_msgs");
-  exportPSAction     = new KAction(i18n("PostScript..."), 0, &DVIRenderer, SLOT(exportPS()), actionCollection(), "export_postscript");
-  exportPDFAction    = new KAction(i18n("PDF..."), 0, &DVIRenderer, SLOT(exportPDF()), actionCollection(), "export_pdf");
-  exportTextAction   = new KAction(i18n("Text..."), 0, this, SLOT(doExportText()), actionCollection(), "export_text");
+  exportPSAction   = new KAction(i18n("PostScript..."), 0, &DVIRenderer, SLOT(exportPS()), actionCollection(), "export_postscript");
+  exportPDFAction  = new KAction(i18n("PDF..."), 0, &DVIRenderer, SLOT(exportPDF()), actionCollection(), "export_pdf");
 
   KStdAction::tipOfDay(this, SLOT(showTip()), actionCollection(), "help_tipofday");
 
@@ -84,6 +85,19 @@ KDVIMultiPage::KDVIMultiPage(QWidget *parentWidget, const char *widgetName, QObj
   QTimer::singleShot(0,this,SLOT(showTipOnStart()));
 }
 
+
+KDVIMultiPage::~KDVIMultiPage()
+{
+  delete docInfoAction;
+  delete embedPSAction;
+  delete exportPSAction;
+  delete exportPDFAction;
+
+  writeSettings();
+  Prefs::writeConfig();
+}
+
+
 KAboutData* KDVIMultiPage::createAboutData()
 {
   KAboutData* about = new KAboutData("kdvi", I18N_NOOP("KDVI"), "1.3",
@@ -107,14 +121,15 @@ KAboutData* KDVIMultiPage::createAboutData()
   return about;
 }
 
-void KDVIMultiPage::slotEmbedPostScript(void)
+
+void KDVIMultiPage::slotEmbedPostScript()
 {
   DVIRenderer.embedPostScript();
   emit askingToCheckActions();
 }
 
 
-void KDVIMultiPage::setEmbedPostScriptAction(void)
+void KDVIMultiPage::setEmbedPostScriptAction()
 {
   if ((DVIRenderer.dviFile == 0) || (DVIRenderer.dviFile->numberOfExternalPSFiles == 0))
     embedPSAction->setEnabled(false);
@@ -148,9 +163,9 @@ void KDVIMultiPage::slotSave()
     fileName = fileName+ending;
 
   if (QFile(fileName).exists()) {
-    int r = KMessageBox::warningYesNo (0, i18n("The file %1\nexists. Do you want to overwrite that file?").arg(fileName),
-                       i18n("Overwrite File"));
-    if (r == KMessageBox::No)
+    int r = KMessageBox::warningContinueCancel (0, i18n("The file %1\nexists. Do you want to overwrite that file?").arg(fileName),
+                       i18n("Overwrite File"), i18n("Overwrite"));
+    if (r == KMessageBox::Cancel)
       return;
   }
 
@@ -171,51 +186,13 @@ void KDVIMultiPage::slotSave_defaultFilename()
 }
 
 
-bool KDVIMultiPage::isModified()
+void KDVIMultiPage::setFile(bool r)
 {
-  if ((DVIRenderer.dviFile == 0) || (DVIRenderer.dviFile->dvi_Data() == 0))
-    return false;
-  else
-    return DVIRenderer.dviFile->isModified;
-}
-
-
-KDVIMultiPage::~KDVIMultiPage()
-{
-  if (timer_id != -1)
-    killTimer(timer_id);
-  timer_id = -1;
-  writeSettings();
-  Prefs::writeConfig();
-  delete printer;
-}
-
-
-bool KDVIMultiPage::openFile()
-{
-  document_history.clear();
-  emit setStatusBarText(i18n("Loading file %1").arg(m_file));
-
-  bool r = DVIRenderer.setFile(m_file);
-  setEmbedPostScriptAction();
-  if (!r)
-    emit setStatusBarText(QString::null);
-
-  //############
-  //An dieser Stelle msste der Zoom-Faktor neu berechnet werden?
-
-  generateDocumentWidgets();
-  emit numberOfPages(DVIRenderer.totalPages());
   enableActions(r);
-
-  QString reference = url().ref();
-  if (!reference.isEmpty())
-    gotoPage(DVIRenderer.parseReference(reference));
-  return r;
 }
 
 
-QStringList KDVIMultiPage::fileFormats()
+QStringList KDVIMultiPage::fileFormats() const
 {
   QStringList r;
   r << i18n("*.dvi *.DVI|TeX Device Independent Files (*.dvi)");
@@ -233,13 +210,13 @@ void KDVIMultiPage::addConfigDialogs(KConfigDialog* configDialog)
   configDialog->addPage(fontConfigWidget, Prefs::self(), i18n("TeX Fonts"), "fonts");
   configDialog->addPage(specialConfigWidget, Prefs::self(), i18n("DVI Specials"), "dvi");
   configDialog->setHelp("preferences", "kdvi");
-  
-  connect(configDialog, SIGNAL(settingsChanged()), this, SLOT(preferencesChanged()));
 }
 
 
 void KDVIMultiPage::preferencesChanged()
 {
+  // Call method from parent class
+  KMultiPage::preferencesChanged();
 #ifdef  KDVI_MULTIPAGE_DEBUG
   kdDebug(4300) << "preferencesChanged" << endl;
 #endif
@@ -251,66 +228,22 @@ void KDVIMultiPage::preferencesChanged()
 }
 
 
-bool KDVIMultiPage::print(const QStringList &pages, int current)
+void KDVIMultiPage::print()
 {
-  // Make sure the KPrinter is available
+  // Obtain a fully initialized KPrinter structure, and disable all
+  // entries in the "Page Size & Placement" tab of the printer dialog.
+  KPrinter *printer = getPrinter(false);
+  // Abort with an error message if no KPrinter could be initialized
   if (printer == 0) {
-    printer = new KPrinter();
-    if (printer == 0) {
-      kdError(4300) << "Could not allocate printer structure" << endl;
-      return false;
-    }
-  }
-  
-  // Feed the printer with useful defaults and information.
-  printer->setPageSelection( KPrinter::ApplicationSide );
-  printer->setCurrentPage( current+1 );
-  printer->setMinMax( 1, DVIRenderer.totalPages() );
-  printer->setFullPage( true );
-
-
-  // If pages are marked, give a list of marked pages to the
-  // printer. We try to be smart and optimize the list by using ranges
-  // ("5-11") wherever possible. The user will be tankful for
-  // that. Complicated? Yeah, but that's life.
-  if (pages.isEmpty() == true)
-    printer->setOption( "kde-range", "" );
-  else {
-    int commaflag = 0;
-    QString range;
-    QStringList::ConstIterator it = pages.begin();
-    do{
-      int val = (*it).toUInt()+1;
-      if (commaflag == 1)
-    range +=  QString(", ");
-      else
-    commaflag = 1;
-      int endval = val;
-      if (it != pages.end()) {
-    QStringList::ConstIterator jt = it;
-    jt++;
-    do{
-      int val2 = (*jt).toUInt()+1;
-      if (val2 == endval+1)
-        endval++;
-      else
-        break;
-      jt++;
-    } while( jt != pages.end() );
-    it = jt;
-      } else
-    it++;
-      if (endval == val)
-    range +=  QString("%1").arg(val);
-      else
-    range +=  QString("%1-%2").arg(val).arg(endval);
-    } while (it != pages.end() );
-    printer->setOption( "kde-range", range );
+    kdError(4300) << "KPrinter not available" << endl;
+    return;
   }
 
-  // Show the printer options requestor
-  if (!printer->setup(scrollView(), i18n("Print %1").arg(m_file.section('/', -1))))
-    return false;
+  // Show the printer options dialog. Return immediately if the user
+  // aborts.
+  if (!printer->setup(parentWdg, i18n("Print %1").arg(m_file.section('/', -1)) ))
+    return;
+
   // This funny method call is necessary for the KPrinter to return
   // proper results in printer->orientation() below. It seems that
   // KPrinter does some options parsing in that method.
@@ -320,7 +253,7 @@ bool KDVIMultiPage::print(const QStringList &pages, int current)
             i18n("The list of pages you selected was empty.\n"
                  "Maybe you made an error in selecting the pages, "
                  "e.g. by giving an invalid range like '7-2'.") );
-    return false;
+    return;
   }
 
   // Turn the results of the options requestor into a list arguments
@@ -431,6 +364,8 @@ bool KDVIMultiPage::print(const QStringList &pages, int current)
   case KPrinter::Tabloid:
     dvips_options += "-t tabloid ";
     break;
+  default:
+    break;
   }
   // Orientation
   if ( printer->orientation() == KPrinter::Landscape )
@@ -460,66 +395,7 @@ bool KDVIMultiPage::print(const QStringList &pages, int current)
   // "True" may be a bit euphemistic. However, since dvips runs
   // concurrently, there is no way of telling the result of the
   // printing command at this stage.
-  return true;
-}
-
-
-// Explanation of the timerEvent.
-//
-// This is a dreadful hack. The problem we adress with this timer
-// event is the following: the kviewshell has a KDirWatch object which
-// looks at the DVI file and calls reload() when the object has
-// changed. That works very nicely in principle, but in practise, when
-// TeX runs for several seconds over a complicated file, this does not
-// work at all. First, most of the time, while TeX is still writing,
-// the file is invalid. Thus, reload() is very often called when the
-// DVI file is bad. We solve this problem by checking the file
-// first. If the file is bad, we do not reload. Second, when the file
-// finally becomes good, it very often happens that KDirWatch does not
-// notify us anymore. Whether this is a bug or a side effect of a
-// feature of KDirWatch, I dare not say. We remedy that problem by
-// using a timer: when reload() was called on a bad file, we
-// automatically come back (via the timerEvent() function) every
-// second and check if the file became good. If so, we stop the
-// timer. It may well happen that KDirWatch calls us several times
-// while we are waiting for the file to become good, but that does not
-// do any harm.
-//
-// -- Stefan Kebekus.
-
-void KDVIMultiPage::timerEvent( QTimerEvent * )
-{
-#ifdef KDVI_MULTIPAGE_DEBUG
-  kdDebug(4300) << "Timer Event " << endl;
-#endif
-  reload();
-}
-
-
-void KDVIMultiPage::reload()
-{
-#ifdef KDVI_MULTIPAGE_DEBUG
-  kdDebug(4300) << "Reload file " << m_file << endl;
-#endif
-  
-  if (DVIRenderer.isValidFile(m_file)) {
-    userSelection.clear();
-    Q_INT32 pg = currentPageNumber();
-
-    killTimer(timer_id);
-    timer_id = -1;
-    bool r = DVIRenderer.setFile(m_file);
-    
-    generateDocumentWidgets();
-    emit numberOfPages(DVIRenderer.totalPages());
-    enableActions(r);
-    emit setStatusBarText(QString::null);
-    markList()->setCurrentPageNumber(pg);
-    emit pageInfo(DVIRenderer.totalPages(), pg );
-  } else {
-    if (timer_id == -1)
-      timer_id = startTimer(1000);
-  }
+  return;
 }
 
 
@@ -530,13 +406,12 @@ void KDVIMultiPage::enableActions(bool b)
   docInfoAction->setEnabled(b);
   exportPSAction->setEnabled(b);
   exportPDFAction->setEnabled(b);
-  exportTextAction->setEnabled(b);
 
   setEmbedPostScriptAction();
 }
 
 
-void KDVIMultiPage::doEnableWarnings(void)
+void KDVIMultiPage::doEnableWarnings()
 {
   KMessageBox::information (scrollView(), i18n("All messages and warnings will now be shown."));
   KMessageBox::enableAllMessages();
@@ -544,38 +419,66 @@ void KDVIMultiPage::doEnableWarnings(void)
 }
 
 
-void KDVIMultiPage::showTip(void)
+void KDVIMultiPage::showTip()
 {
   KTipDialog::showTip(scrollView(), "kdvi/tips", true);
 }
 
 
-void KDVIMultiPage::showTipOnStart(void)
+void KDVIMultiPage::showTipOnStart()
 {
   KTipDialog::showTip(scrollView(), "kdvi/tips");
 }
 
 
-documentWidget* KDVIMultiPage::createDocumentWidget()
+DocumentWidget* KDVIMultiPage::createDocumentWidget()
 {
-  QSize rendererSuggestedSize = pageCache.sizeOfPageInPixel(/*page+*/1); // FIXME
-  QSize widgetSize;
-  if (rendererSuggestedSize.isEmpty()) {
-    widgetSize.setWidth(100);
-    widgetSize.setHeight(100);
-  } else
-    widgetSize = rendererSuggestedSize;
+  DVIWidget* documentWidget = new DVIWidget(scrollView()->viewport(), scrollView(), pageCache,
+                              "singlePageWidget" );
 
-
-  // TODO: handle different sizes per page.
-  DVIWidget* documentWidget = new DVIWidget(scrollView()->viewport(), scrollView(), widgetSize, &pageCache,
-                              &userSelection, "singlePageWidget" );
+  // Lets not forget the connections we make in the KMultiPage
+  connect(documentWidget, SIGNAL(clearSelection()), this, SLOT(clearSelection()));
+  connect(this, SIGNAL(enableMoveTool(bool)), documentWidget, SLOT(slotEnableMoveTool(bool)));
 
   // Handle source links
-  connect(documentWidget, SIGNAL(SRCLink(const QString&,QMouseEvent *, documentWidget *)), getRenderer(),
-          SLOT(handleSRCLink(const QString &,QMouseEvent *, documentWidget *)));
+  connect(documentWidget, SIGNAL(SRCLink(const QString&, QMouseEvent*, DocumentWidget*)), getRenderer(),
+          SLOT(handleSRCLink(const QString& ,QMouseEvent*, DocumentWidget*)));
 
   return documentWidget;
 }
 
+
+void KDVIMultiPage::initializePageCache()
+{
+  pageCache = new DVIPageCache();
+}
+
+
+void KDVIMultiPage::showFindTextDialog()
+{
+  if ((getRenderer().isNull()) || (getRenderer()->supportsTextSearch() == false))
+    return;
+
+  if (!searchUsed)
+  {
+    // WARNING: This text appears several times in the code. Change
+    // everywhere, or nowhere!
+    if (KMessageBox::warningContinueCancel( scrollView(), 
+					    i18n("<qt>This function searches the DVI file for plain text. Unfortunately, this version of "
+						 "KDVI treats only plain ASCII characters properly. Symbols, ligatures, mathematical "
+						 "formulae, accented characters, and non-English text, such as Russian or Korean, will "
+						 "most likely be messed up completely. Continue anyway?</qt>"),
+					    i18n("Function May Not Work as Expected"),
+					    KStdGuiItem::cont(),
+					    "warning_search_text_may_not_work") == KMessageBox::Cancel)
+      return;
+
+    // Remember that we don't need to show the warning message again.
+    searchUsed = true;
+  }
+
+  // Now really show the search widget
+  KMultiPage::showFindTextDialog();
+}
+
 #include "kdvi_multipage.moc"
