@@ -1,32 +1,36 @@
 
 // special.cpp
 
-// Methods for dviwin which deal with "\special" commands found in the
+// Methods for dviRenderer which deal with "\special" commands found in the
 // DVI file
 
 // Copyright 2000--2004, Stefan Kebekus (kebekus@kde.org).
 
+#include <config.h>
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kmimetype.h>
 #include <kprocio.h>
-#include <kstringhandler.h>
 #include <qdir.h>
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qimage.h>
 #include <qstringlist.h>
 
-#include "dviwin.h"
+#include "dviFile.h"
+#include "dviRenderer.h"
+#include "hyperlink.h"
 #include "kdvi.h"
 #include "kdvi_multipage.h"
+#include "psgs.h"
 #include "xdvi.h"
 
 //#define DEBUG_SPECIAL
 
-extern QPainter foreGroundPaint;
+extern QPainter *foreGroundPainter;
 
-void dviRenderer::printErrorMsgForSpecials(QString msg)
+void dviRenderer::printErrorMsgForSpecials(const QString& msg)
 {
   if (dviFile->errorCounter < 25) {
     kdError(4300) << msg << endl;
@@ -40,22 +44,94 @@ void dviRenderer::printErrorMsgForSpecials(QString msg)
 // dvips. If the spec could not be parsed, an invalid color will be
 // returned.
 
-QColor parseColorSpecification(QString colorSpec)
+QColor dviRenderer::parseColorSpecification(const QString& colorSpec)
 {
-  QString specType = KStringHandler::word(colorSpec, (unsigned int)0);
+  // Initialize the map of known colors, if that is not done yet.
+  if (namedColors.isEmpty()) {
+    namedColors["Red"] = QColor( (int)(255.0*1), (int)(255.0*0), (int)(255.0*0));
+    namedColors["Tan"] = QColor( (int)(255.0*0.86), (int)(255.0*0.58), (int)(255.0*0.44));
+    namedColors["Blue"] = QColor( (int)(255.0*0), (int)(255.0*0), (int)(255.0*1));
+    namedColors["Cyan"] = QColor( (int)(255.0*0), (int)(255.0*1), (int)(255.0*1));
+    namedColors["Gray"] = QColor( (int)(255.0*0.5), (int)(255.0*0.5), (int)(255.0*0.5));
+    namedColors["Plum"] = QColor( (int)(255.0*0.5), (int)(255.0*0), (int)(255.0*1));
+    namedColors["Black"] = QColor( (int)(255.0*0), (int)(255.0*0), (int)(255.0*0));
+    namedColors["Brown"] = QColor( (int)(255.0*0.4), (int)(255.0*0), (int)(255.0*0));
+    namedColors["Green"] = QColor( (int)(255.0*0), (int)(255.0*1), (int)(255.0*0));
+    namedColors["Melon"] = QColor( (int)(255.0*1), (int)(255.0*0.54), (int)(255.0*0.5));
+    namedColors["Peach"] = QColor( (int)(255.0*1), (int)(255.0*0.5), (int)(255.0*0.3));
+    namedColors["Sepia"] = QColor( (int)(255.0*0.3), (int)(255.0*0), (int)(255.0*0));
+    namedColors["White"] = QColor( (int)(255.0*1), (int)(255.0*1), (int)(255.0*1));
+    namedColors["Maroon"] = QColor( (int)(255.0*0.68), (int)(255.0*0), (int)(255.0*0));
+    namedColors["Orange"] = QColor( (int)(255.0*1), (int)(255.0*0.39), (int)(255.0*0.13));
+    namedColors["Orchid"] = QColor( (int)(255.0*0.68), (int)(255.0*0.36), (int)(255.0*1));
+    namedColors["Purple"] = QColor( (int)(255.0*0.55), (int)(255.0*0.14), (int)(255.0*1));
+    namedColors["Salmon"] = QColor( (int)(255.0*1), (int)(255.0*0.47), (int)(255.0*0.62));
+    namedColors["Violet"] = QColor( (int)(255.0*0.21), (int)(255.0*0.12), (int)(255.0*1));
+    namedColors["Yellow"] = QColor( (int)(255.0*1), (int)(255.0*1), (int)(255.0*0));
+    namedColors["Apricot"] = QColor( (int)(255.0*1), (int)(255.0*0.68), (int)(255.0*0.48));
+    namedColors["Emerald"] = QColor( (int)(255.0*0), (int)(255.0*1), (int)(255.0*0.5));
+    namedColors["Fuchsia"] = QColor( (int)(255.0*0.45), (int)(255.0*0.01), (int)(255.0*0.92));
+    namedColors["Magenta"] = QColor( (int)(255.0*1), (int)(255.0*0), (int)(255.0*1));
+    namedColors["SkyBlue"] = QColor( (int)(255.0*0.38), (int)(255.0*1), (int)(255.0*0.88));
+    namedColors["Thistle"] = QColor( (int)(255.0*0.88), (int)(255.0*0.41), (int)(255.0*1));
+    namedColors["BrickRed"] = QColor( (int)(255.0*0.72), (int)(255.0*0), (int)(255.0*0));
+    namedColors["Cerulean"] = QColor( (int)(255.0*0.06), (int)(255.0*0.89), (int)(255.0*1));
+    namedColors["Lavender"] = QColor( (int)(255.0*1), (int)(255.0*0.52), (int)(255.0*1));
+    namedColors["Mahogany"] = QColor( (int)(255.0*0.65), (int)(255.0*0), (int)(255.0*0));
+    namedColors["Mulberry"] = QColor( (int)(255.0*0.64), (int)(255.0*0.08), (int)(255.0*0.98));
+    namedColors["NavyBlue"] = QColor( (int)(255.0*0.06), (int)(255.0*0.46), (int)(255.0*1));
+    namedColors["SeaGreen"] = QColor( (int)(255.0*0.31), (int)(255.0*1), (int)(255.0*0.5));
+    namedColors["TealBlue"] = QColor( (int)(255.0*0.12), (int)(255.0*0.98), (int)(255.0*0.64));
+    namedColors["BlueGreen"] = QColor( (int)(255.0*0.15), (int)(255.0*1), (int)(255.0*0.67));
+    namedColors["CadetBlue"] = QColor( (int)(255.0*0.38), (int)(255.0*0.43), (int)(255.0*0.77));
+    namedColors["Dandelion"] = QColor( (int)(255.0*1), (int)(255.0*0.71), (int)(255.0*0.16));
+    namedColors["Goldenrod"] = QColor( (int)(255.0*1), (int)(255.0*0.9), (int)(255.0*0.16));
+    namedColors["LimeGreen"] = QColor( (int)(255.0*0.5), (int)(255.0*1), (int)(255.0*0));
+    namedColors["OrangeRed"] = QColor( (int)(255.0*1), (int)(255.0*0), (int)(255.0*0.5));
+    namedColors["PineGreen"] = QColor( (int)(255.0*0), (int)(255.0*0.75), (int)(255.0*0.16));
+    namedColors["RawSienna"] = QColor( (int)(255.0*0.55), (int)(255.0*0), (int)(255.0*0));
+    namedColors["RedOrange"] = QColor( (int)(255.0*1), (int)(255.0*0.23), (int)(255.0*0.13));
+    namedColors["RedViolet"] = QColor( (int)(255.0*0.59), (int)(255.0*0), (int)(255.0*0.66));
+    namedColors["Rhodamine"] = QColor( (int)(255.0*1), (int)(255.0*0.18), (int)(255.0*1));
+    namedColors["RoyalBlue"] = QColor( (int)(255.0*0), (int)(255.0*0.5), (int)(255.0*1));
+    namedColors["RubineRed"] = QColor( (int)(255.0*1), (int)(255.0*0), (int)(255.0*0.87));
+    namedColors["Turquoise"] = QColor( (int)(255.0*0.15), (int)(255.0*1), (int)(255.0*0.8));
+    namedColors["VioletRed"] = QColor( (int)(255.0*1), (int)(255.0*0.19), (int)(255.0*1));
+    namedColors["Aquamarine"] = QColor( (int)(255.0*0.18), (int)(255.0*1), (int)(255.0*0.7));
+    namedColors["BlueViolet"] = QColor( (int)(255.0*0.1), (int)(255.0*0.05), (int)(255.0*0.96));
+    namedColors["DarkOrchid"] = QColor( (int)(255.0*0.6), (int)(255.0*0.2), (int)(255.0*0.8));
+    namedColors["OliveGreen"] = QColor( (int)(255.0*0), (int)(255.0*0.6), (int)(255.0*0));
+    namedColors["Periwinkle"] = QColor( (int)(255.0*0.43), (int)(255.0*0.45), (int)(255.0*1));
+    namedColors["Bittersweet"] = QColor( (int)(255.0*0.76), (int)(255.0*0.01), (int)(255.0*0));
+    namedColors["BurntOrange"] = QColor( (int)(255.0*1), (int)(255.0*0.49), (int)(255.0*0));
+    namedColors["ForestGreen"] = QColor( (int)(255.0*0), (int)(255.0*0.88), (int)(255.0*0));
+    namedColors["GreenYellow"] = QColor( (int)(255.0*0.85), (int)(255.0*1), (int)(255.0*0.31));
+    namedColors["JungleGreen"] = QColor( (int)(255.0*0.01), (int)(255.0*1), (int)(255.0*0.48));
+    namedColors["ProcessBlue"] = QColor( (int)(255.0*0.04), (int)(255.0*1), (int)(255.0*1));
+    namedColors["RoyalPurple"] = QColor( (int)(255.0*0.25), (int)(255.0*0.1), (int)(255.0*1));
+    namedColors["SpringGreen"] = QColor( (int)(255.0*0.74), (int)(255.0*1), (int)(255.0*0.24));
+    namedColors["YellowGreen"] = QColor( (int)(255.0*0.56), (int)(255.0*1), (int)(255.0*0.26));
+    namedColors["MidnightBlue"] = QColor( (int)(255.0*0), (int)(255.0*0.44), (int)(255.0*0.57));
+    namedColors["YellowOrange"] = QColor( (int)(255.0*1), (int)(255.0*0.58), (int)(255.0*0));
+    namedColors["CarnationPink"] = QColor( (int)(255.0*1), (int)(255.0*0.37), (int)(255.0*1));
+    namedColors["CornflowerBlue"] = QColor( (int)(255.0*0.35), (int)(255.0*0.87), (int)(255.0*1));
+    namedColors["WildStrawberry"] = QColor( (int)(255.0*1), (int)(255.0*0.04), (int)(255.0*0.61));
+  }
+  
+  QString specType = colorSpec.section(' ', 0, 0);
 
   if (specType.find("rgb", false) == 0) {
     bool ok;
 
-    double r = KStringHandler::word(colorSpec, (unsigned int)1).toDouble(&ok);
+    double r = colorSpec.section(' ', 1, 1).toDouble(&ok);
     if ((ok == false) || (r < 0.0) || (r > 1.0))
       return QColor();
     
-    double g = KStringHandler::word(colorSpec, (unsigned int)2).toDouble(&ok);
+    double g = colorSpec.section(' ', 2, 2).toDouble(&ok);
     if ((ok == false) || (g < 0.0) || (g > 1.0))
       return QColor();
     
-    double b = KStringHandler::word(colorSpec, (unsigned int)3).toDouble(&ok);
+    double b = colorSpec.section(' ', 3, 3).toDouble(&ok);
     if ((ok == false) || (b < 0.0) || (b > 1.0))
       return QColor();
 
@@ -65,15 +141,15 @@ QColor parseColorSpecification(QString colorSpec)
   if (specType.find("hsb", false) == 0) {
     bool ok;
 
-    double h = KStringHandler::word(colorSpec, (unsigned int)1).toDouble(&ok);
+    double h = colorSpec.section(' ', 1, 1).toDouble(&ok);
     if ((ok == false) || (h < 0.0) || (h > 1.0))
       return QColor();
     
-    double s = KStringHandler::word(colorSpec, (unsigned int)2).toDouble(&ok);
+    double s = colorSpec.section(' ', 2, 2).toDouble(&ok);
     if ((ok == false) || (s < 0.0) || (s > 1.0))
       return QColor();
     
-    double b = KStringHandler::word(colorSpec, (unsigned int)3).toDouble(&ok);
+    double b = colorSpec.section(' ', 3, 3).toDouble(&ok);
     if ((ok == false) || (b < 0.0) || (b > 1.0))
       return QColor();
 
@@ -83,19 +159,19 @@ QColor parseColorSpecification(QString colorSpec)
   if (specType.find("cmyk", false) == 0) {
     bool ok;
 
-    double c = KStringHandler::word(colorSpec, (unsigned int)1).toDouble(&ok);
+    double c = colorSpec.section(' ', 1, 1).toDouble(&ok);
     if ((ok == false) || (c < 0.0) || (c > 1.0))
       return QColor();
     
-    double m = KStringHandler::word(colorSpec, (unsigned int)2).toDouble(&ok);
+    double m = colorSpec.section(' ', 2, 2).toDouble(&ok);
     if ((ok == false) || (m < 0.0) || (m > 1.0))
       return QColor();
     
-    double y = KStringHandler::word(colorSpec, (unsigned int)3).toDouble(&ok);
+    double y = colorSpec.section(' ', 3, 3).toDouble(&ok);
     if ((ok == false) || (y < 0.0) || (y > 1.0))
       return QColor();
     
-    double k = KStringHandler::word(colorSpec, (unsigned int)3).toDouble(&ok);
+    double k = colorSpec.section(' ', 3, 3).toDouble(&ok);
     if ((ok == false) || (k < 0.0) || (k > 1.0))
       return QColor();
 
@@ -116,13 +192,18 @@ QColor parseColorSpecification(QString colorSpec)
   if (specType.find("gray", false) == 0) {
     bool ok;
 
-    double g = KStringHandler::word(colorSpec, (unsigned int)1).toDouble(&ok);
+    double g = colorSpec.section(' ', 1, 1).toDouble(&ok);
     if ((ok == false) || (g < 0.0) || (g > 1.0))
       return QColor();
     
     return QColor((int)(g*255.0+0.5), (int)(g*255.0+0.5), (int)(g*255.0+0.5));
   }
 
+  // Check if the color is one of the known named colors.
+  QMap<QString, QColor>::Iterator f = namedColors.find(specType);
+  if (f != namedColors.end())
+    return *f;
+  
   return QColor(specType);
 }
 
@@ -131,11 +212,11 @@ QColor parseColorSpecification(QString colorSpec)
 
 
 
-void dviRenderer::color_special(QString cp)
+void dviRenderer::color_special(const QString& _cp)
 {
-  cp = cp.stripWhiteSpace();
+  QString const cp = _cp.stripWhiteSpace();
   
-  QString command = KStringHandler::word(cp, (unsigned int)0);
+  QString command = cp.section(' ', 0, 0);
   
   if (command == "pop") {
     // Take color off the stack
@@ -149,7 +230,7 @@ void dviRenderer::color_special(QString cp)
   
   if (command == "push") {
     // Get color specification
-    QColor col = parseColorSpecification(KStringHandler::word(cp, "1:"));
+    QColor const col = parseColorSpecification(cp.section(' ', 1));
     // Set color
     if (col.isValid()) 
       colorStack.push(col); 
@@ -170,8 +251,9 @@ void dviRenderer::color_special(QString cp)
 }
 
 
-void dviRenderer::html_href_special(QString cp)
+void dviRenderer::html_href_special(const QString& _cp)
 {
+  QString cp = _cp;
   cp.truncate(cp.find('"'));
   
 #ifdef DEBUG_SPECIAL
@@ -181,7 +263,7 @@ void dviRenderer::html_href_special(QString cp)
 }
 
 
-void dviRenderer::html_anchor_end(void)
+void dviRenderer::html_anchor_end()
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "HTML-special, anchor-end" << endl;
@@ -194,7 +276,7 @@ void dviRenderer::html_anchor_end(void)
 }
 
 
-void dviRenderer::source_special(QString cp)
+void dviRenderer::source_special(const QString& cp)
 {
   // only when rendering really takes place: set source_href to the
   // current special string. When characters are rendered, the
@@ -208,25 +290,30 @@ void dviRenderer::source_special(QString cp)
 }
 
 
-void parse_special_argument(QString strg, const char *argument_name, int *variable)
+void parse_special_argument(const QString& strg, const char* argument_name, int* variable)
 {
-  bool    OK;
-  
   int index = strg.find(argument_name);
   if (index >= 0) {
-    QString tmp     = strg.mid(index + strlen(argument_name));
-    tmp.truncate(tmp.find(' '));
-    float tmp_float = tmp.toFloat(&OK);
+    QString tmp = strg.mid(index + strlen(argument_name));
+    index = tmp.find(' ');
+    if (index >= 0)
+      tmp.truncate(index);
+
+    bool OK;
+    float const tmp_float = tmp.toFloat(&OK);
+
     if (OK)
-      *variable = (int)(tmp_float+0.5);
+      *variable = int(tmp_float+0.5);
     else
       // Maybe we should open a dialog here.
-      kdError(4300) << i18n("Malformed parameter in the epsf special command.") << endl;
+      kdError(4300) << i18n("Malformed parameter in the epsf special command.\n"
+                            "Expected a float to follow %1 in %2")
+                       .arg(argument_name).arg(strg) << endl;
   }
 }
 
 
-void dviRenderer::epsf_special(QString cp)
+void dviRenderer::epsf_special(const QString& cp)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "epsf-special: psfile=" << cp <<endl;
@@ -249,7 +336,7 @@ void dviRenderer::epsf_special(QString cp)
   if ((EPSfilename.at(0) == '\"') && (EPSfilename.at(EPSfilename.length()-1) == '\"')) {
     EPSfilename = EPSfilename.mid(1,EPSfilename.length()-2);
   }
-  EPSfilename = ghostscript_interface::locateEPSfile(EPSfilename, dviFile);
+  EPSfilename = ghostscript_interface::locateEPSfile(EPSfilename, baseURL);
   
   // Now parse the arguments. 
   int  llx     = 0; 
@@ -271,14 +358,13 @@ void dviRenderer::epsf_special(QString cp)
   parse_special_argument(include_command, "rhi=", &rhi);
   parse_special_argument(include_command, "angle=", &angle);
 
-
-  // If the file name ends in 'png', 'gif', 'jpg' or 'jpeg', we assume
-  // that this is NOT a PostScript file, and we need to draw that
-  // here.
-  QString ending = EPSfilename.section('.', -1).lower();
-  bool isGFX = false;
-  if ((ending == "png") || (ending == "gif") || (ending == "jpg") || (ending == "jpeg") || (ending == "mng"))
-    isGFX = true;
+  // If we have a png, gif, jpeg or mng file, we need to draw it here.
+  KMimeType::Ptr const mime_type = KMimeType::findByFileContent(EPSfilename);
+  QString const & mime_type_name = mime_type->name();
+  bool const isGFX = (mime_type_name == "image/png" ||
+                      mime_type_name == "image/gif" ||
+                      mime_type_name == "image/jpeg" ||
+                      mime_type_name == "video/x-mng");
 
   // So, if we do not have a PostScript file, but a graphics file, and
   // if that file exists, we draw it here.
@@ -303,8 +389,8 @@ void dviRenderer::epsf_special(QString cp)
     bbox_height *= 0.1 * 65536.0*fontPixelPerDVIunit / shrinkfactor;
     
     QImage image(EPSfilename);
-    image = image.smoothScale(bbox_width, bbox_height);
-    foreGroundPaint.drawImage( ((int) ((currinf.data.dvi_h) / (shrinkfactor * 65536))), currinf.data.pxl_v - (int)bbox_height, image);
+    image = image.smoothScale((int)(bbox_width), (int)(bbox_height));
+    foreGroundPainter->drawImage( ((int) ((currinf.data.dvi_h) / (shrinkfactor * 65536))), currinf.data.pxl_v - (int)bbox_height, image);
     return;
   }
 
@@ -330,26 +416,28 @@ void dviRenderer::epsf_special(QString cp)
     
     QRect bbox(((int) ((currinf.data.dvi_h) / (shrinkfactor * 65536))), currinf.data.pxl_v - (int)bbox_height,
 	       (int)bbox_width, (int)bbox_height);
-    foreGroundPaint.save();
+
+    foreGroundPainter->save();
+
     if (QFile::exists(EPSfilename))
-      foreGroundPaint.setBrush(Qt::lightGray);
+      foreGroundPainter->setBrush(Qt::lightGray);
     else
-      foreGroundPaint.setBrush(Qt::red);
-    foreGroundPaint.setPen(Qt::black);
-    foreGroundPaint.drawRoundRect(bbox, 2, 2);
+      foreGroundPainter->setBrush(Qt::red);
+    foreGroundPainter->setPen(Qt::black);
+    foreGroundPainter->drawRoundRect(bbox, 2, 2);
     if (QFile::exists(EPSfilename))
-      foreGroundPaint.drawText (bbox, (int)(Qt::AlignCenter), EPSfilename, -1);
+      foreGroundPainter->drawText (bbox, (int)(Qt::AlignCenter), EPSfilename, -1);
     else
-      foreGroundPaint.drawText (bbox, (int)(Qt::AlignCenter), 
+      foreGroundPainter->drawText (bbox, (int)(Qt::AlignCenter), 
 				i18n("File not found: \n %1").arg(EPSfilename), -1);
-    foreGroundPaint.restore();
+    foreGroundPainter->restore();
   }
   
   return;
 }
 
 
-void dviRenderer::TPIC_flushPath_special(void)
+void dviRenderer::TPIC_flushPath_special()
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "TPIC special flushPath" << endl;
@@ -361,13 +449,13 @@ void dviRenderer::TPIC_flushPath_special(void)
   }
 
   QPen pen(Qt::black, (int)(penWidth_in_mInch*resolutionInDPI/1000.0 + 0.5));  // Sets the pen size in milli-inches
-  foreGroundPaint.setPen(pen);
-  foreGroundPaint.drawPolyline(TPIC_path, 0, number_of_elements_in_path);
+  foreGroundPainter->setPen(pen);
+  foreGroundPainter->drawPolyline(TPIC_path, 0, number_of_elements_in_path);
   number_of_elements_in_path = 0;
 }
 
 
-void dviRenderer::TPIC_addPath_special(QString cp)
+void dviRenderer::TPIC_addPath_special(const QString& cp)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "TPIC special addPath: " << cp << endl;
@@ -376,19 +464,19 @@ void dviRenderer::TPIC_addPath_special(QString cp)
   // Adds a point to the path list
   QString cp_noWhiteSpace = cp.stripWhiteSpace();
   bool ok;
-  float xKoord = KStringHandler::word(cp_noWhiteSpace, (uint)0).toFloat(&ok);
+  float xKoord = cp_noWhiteSpace.section(' ', 0, 0).toFloat(&ok);
   if (ok == false) {
     printErrorMsgForSpecials( QString("TPIC special; cannot parse first argument in 'pn %1'.").arg(cp) );
     return;
   }
-  float yKoord = KStringHandler::word(cp_noWhiteSpace, (uint)1).toFloat(&ok);
+  float yKoord = cp_noWhiteSpace.section(' ', 1, 1).toFloat(&ok);
   if (ok == false) {
     printErrorMsgForSpecials( QString("TPIC special; cannot parse second argument in 'pn %1'.").arg(cp) );
     return;
   }
   
   float mag = dviFile->getMagnification()/1000.0;
-  
+
   int x = (int)( currinf.data.dvi_h/(shrinkfactor*65536.0) + mag*xKoord*resolutionInDPI/1000.0 + 0.5 );
   int y = (int)( currinf.data.pxl_v + mag*yKoord*resolutionInDPI/1000.0 + 0.5 );
   
@@ -401,7 +489,7 @@ void dviRenderer::TPIC_addPath_special(QString cp)
 }
 
 
-void dviRenderer::TPIC_setPen_special(QString cp)
+void dviRenderer::TPIC_setPen_special(const QString& cp)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "TPIC special setPen: " << cp << endl;
@@ -581,11 +669,11 @@ void dviRenderer::applicationDoSpecial(char *cp)
       int x = ((int) ((currinf.data.dvi_h) / (shrinkfactor * 65536)));
       int y = currinf.data.pxl_v;
       
-      foreGroundPaint.save();
+      foreGroundPainter->save();
       // Rotate about the current point
-      foreGroundPaint.translate(x,y);
-      foreGroundPaint.rotate(-angle);
-      foreGroundPaint.translate(-x,-y);
+      foreGroundPainter->translate(x,y);
+      foreGroundPainter->rotate(-angle);
+      foreGroundPainter->translate(-x,-y);
     } else
       printErrorMsgForSpecials( i18n("Error in DVIfile '%1', page %2. Could not interpret angle in text rotation special." ).
 				arg(dviFile->filename).arg(current_page));
@@ -594,7 +682,7 @@ void dviRenderer::applicationDoSpecial(char *cp)
   // The graphicx package marks the end of rotated text with this
   // special. The state of the painter is restored.
   if (special_command == "ps: currentpoint grestore moveto") {
-    foreGroundPaint.restore();
+    foreGroundPainter->restore();
   }
   
   // The following special commands are not used here; they are of
