@@ -1,6 +1,6 @@
    /*
 
-    $Id: kfax.cpp 378907 2005-01-16 00:28:57Z deller $
+    $Id: kfax.cpp 465369 2005-09-29 14:33:08Z mueller $
 
     Copyright (C) 1997 Bernd Johannes Wuebben
                        wuebben@math.cornell.edu
@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     */
 
