@@ -1,5 +1,5 @@
 /*
-** $Id: gif-info.c 212185 2003-03-07 22:11:39Z waba $
+** $Id: gif-info.c 465369 2005-09-29 14:33:08Z mueller $
 **
 ** Minimal GIF parser, for use in extracting and setting metadata.
 ** Modified for standalone & KDE calling by Bryce Nesbitt
@@ -50,7 +50,7 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **
 */
 
@@ -105,7 +105,7 @@ static long int pos;
 
 static char     true[] = "True";
 static char     false[] = "False";
-static char     id[] = "$Id: gif-info.c 212185 2003-03-07 22:11:39Z waba $";
+static char     id[] = "$Id: gif-info.c 465369 2005-09-29 14:33:08Z mueller $";
 
 /*****************************************************************************/
 
