@@ -1,6 +1,6 @@
 
 /*
-   Copyright (c) 2003-2004 Clarence Dang <dang@kde.org>
+   Copyright (c) 2003,2004,2005 Clarence Dang <dang@kde.org>
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -56,7 +56,6 @@ kpViewManager::kpViewManager (kpMainWindow *mainWindow)
       m_selectionBorderVisible (false),
       m_selectionBorderFinished (false)
 {
-
     m_queueUpdatesCounter = m_fastUpdatesCounter =  0;
 }
 
@@ -320,7 +319,7 @@ int kpViewManager::textCursorCol () const
     return m_textCursorCol;
 }
 
-void kpViewManager::setTextCursorPosition (int row, int col)
+void kpViewManager::setTextCursorPosition (int row, int col, bool isUpdateMicroFocusHint)
 {
     if (row == m_textCursorRow && col == m_textCursorCol)
         return;
@@ -339,6 +338,26 @@ void kpViewManager::setTextCursorPosition (int row, int col)
 
     restoreQueueUpdates ();
     restoreFastUpdates ();
+
+    if (isUpdateMicroFocusHint)
+    {
+        kpDocument *doc = m_mainWindow->document ();
+        if (!doc)
+            return;
+        
+        kpSelection *sel = doc->selection ();
+        if (!sel || !sel->isText ())
+            return;
+
+        if (m_viewUnderCursor)
+        {
+            QPoint topLeft = sel->pointForTextRowCol (m_textCursorRow, m_textCursorCol);
+            if (topLeft != KP_INVALID_POINT)
+            {
+                m_viewUnderCursor->updateMicroFocusHint(QRect (topLeft.x (), topLeft.y (), 1, sel->textStyle ().fontMetrics ().height ())); 
+            }
+        }
+    }
 }
 
 
@@ -496,6 +515,21 @@ void kpViewManager::setViewUnderCursor (kpView *view)
 }
 
 
+// public
+kpView *kpViewManager::activeView () const
+{
+    for (QPtrList <kpView>::const_iterator it = m_views.begin ();
+         it != m_views.end ();
+         it++)
+    {
+        if ((*it)->isActiveWindow ())
+            return (*it);
+    }
+
+    return 0;
+}
+
+
 // public
 bool kpViewManager::queueUpdates () const
 {
@@ -606,6 +640,37 @@ void kpViewManager::updateView (kpView *v, const QRegion &viewRegion)
         v->addToQueuedArea (viewRegion);
 }
 
+void kpViewManager::updateViewRectangleEdges (kpView *v, const QRect &viewRect)
+{
+    if (viewRect.height () <= 0 || viewRect.width () <= 0)
+        return;
+
+    // Top line
+    updateView (v, QRect (viewRect.x (), viewRect.y (),
+                          viewRect.width (), 1));
+
+    if (viewRect.height () >= 2)
+    {
+        // Bottom line
+        updateView (v, QRect (viewRect.x (), viewRect.bottom (),
+                              viewRect.width (), 1));
+
+        if (viewRect.height () > 2)
+        {
+            // Left line
+            updateView (v, QRect (viewRect.x (), viewRect.y () + 1,
+                                  1, viewRect.height () - 2));
+
+            if (viewRect.width () >= 2)
+            {
+                // Right line
+                updateView (v, QRect (viewRect.right (), viewRect.y () + 1,
+                                      1, viewRect.height () - 2));
+            }
+        }
+    }
+}
+
 
 void kpViewManager::updateViews ()
 {
@@ -632,13 +697,13 @@ void kpViewManager::updateViews (const QRect &docRect)
         if (view->zoomLevelX () % 100 == 0 && view->zoomLevelY () % 100 == 0)
         {
         #if DEBUG_KP_VIEW_MANAGER && 0
-            kdDebug () << "\t\tviewRect=" << view->zoomDocToView (docRect) << endl;
+            kdDebug () << "\t\tviewRect=" << view->transformDocToView (docRect) << endl;
         #endif
-            updateView (view, view->zoomDocToView (docRect));
+            updateView (view, view->transformDocToView (docRect));
         }
         else
         {
-            QRect viewRect = view->zoomDocToView (docRect);
+            QRect viewRect = view->transformDocToView (docRect);
 
             int diff = qRound (double (QMAX (view->zoomLevelX (), view->zoomLevelY ())) / 100.0) + 1;
 
@@ -661,11 +726,11 @@ void kpViewManager::updateViews (int x, int y, int w, int h)
     updateViews (QRect (x, y, w, h));
 }
 
-void kpViewManager::resizeViews (int docWidth, int docHeight)
+
+void kpViewManager::adjustViewsToEnvironment ()
 {
 #if DEBUG_KP_VIEW_MANAGER && 1
-    kdDebug () << "kpViewManager::resizeViews(" << docWidth << ","
-               << docHeight << ")"
+    kdDebug () << "kpViewManager::adjustViewsToEnvironment()"
                << " numViews=" << m_views.count ()
                << endl;
 #endif
@@ -676,19 +741,10 @@ void kpViewManager::resizeViews (int docWidth, int docHeight)
         kpView *view = *it;
 
     #if DEBUG_KP_VIEW_MANAGER && 1
-        kdDebug () << "\tresize view: " << view->name ()
-                   << " (variableZoom=" << view->hasVariableZoom () << ")"
+        kdDebug () << "\tview: " << view->name ()
                    << endl;
     #endif
-        if (view->hasVariableZoom ())
-        {
-            view->slotUpdateVariableZoom ();
-        }
-        else
-        {
-            view->resize (view->zoomDocToViewX (docWidth),
-                          view->zoomDocToViewY (docHeight));
-        }
+        view->adjustToEnvironment ();
     }
 }
 
