@@ -31,7 +31,6 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmenubar.h>
-#include <kpopupmenu.h>
 #include <kparts/componentfactory.h>
 #include <kio/netaccess.h>
 #include <kmainwindowiface.h>
@@ -89,7 +88,8 @@ void Shell::init()
   }
   connect( this, SIGNAL( restoreDocument(const KURL &, int) ),m_part, SLOT( restoreDocument(const KURL &, int)));
   connect( this, SIGNAL( saveDocumentRestoreInfo(KConfig*) ), m_part, SLOT( saveDocumentRestoreInfo(KConfig*)));
-
+  connect( m_part, SIGNAL( enablePrintAction(bool) ), m_printAction, SLOT( setEnabled(bool)));
+  
   readSettings();
   if (!KGlobal::config()->hasGroup("MainWindow"))
   {
@@ -97,14 +97,14 @@ void Shell::init()
     kmwi.maximize();
   }
   setAutoSaveSettings();
-
+  
   if (m_openUrl.isValid()) QTimer::singleShot(0, this, SLOT(delayedOpen()));
 }
 
-void Shell::delayedOpen() 
-{ 
-  openURL(m_openUrl); 
-} 
+void Shell::delayedOpen()
+{
+   openURL(m_openUrl);
+}
 
 Shell::~Shell()
 {
@@ -211,7 +211,7 @@ Shell::fileOpen()
   // this slot is called whenever the File->Open menu is selected,
   // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
   // button is clicked
-    KURL url = KFileDialog::getOpenURL( QString::null, "application/pdf" );//getOpenFileName();
+    KURL url = KFileDialog::getOpenURL( QString::null, "application/pdf application/postscript" );//getOpenFileName();
 
   if (!url.isEmpty())
     openURL(url);
