@@ -43,7 +43,7 @@ void PMConeEdit::createTopWidgets( )
    m_pEnd2 = new PMVectorEdit( "x", "y", "z", this );
    m_pRadius1 = new PMFloatEdit( this );
    m_pRadius2 = new PMFloatEdit( this );   
-   m_pOpen = new QCheckBox( i18n( "Open" ), this );
+   m_pOpen = new QCheckBox( i18n( "type of the object", "Open" ), this );
 
    gl = new QGridLayout( topLayout( ), 2, 2 );
    gl->addWidget( new QLabel( i18n( "End 1:" ), this ), 0, 0 );
