@@ -43,7 +43,7 @@ PMViewStructure* PMCylinder::s_pDefaultViewStructure = 0;
 int PMCylinder::s_numSteps = c_defaultCylinderSteps;
 int PMCylinder::s_parameterKey = 0;
 
-PMDefinePropertyClass( PMCylinder, PMProperty );
+PMDefinePropertyClass( PMCylinder, PMCylinderProperty );
 
 PMMetaObject* PMCylinder::s_pMetaObject = 0;
 PMObject* createNewCylinder( PMPart* part )
@@ -104,13 +104,13 @@ PMMetaObject* PMCylinder::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Cylinder", Base::metaObject( ),
                                         createNewCylinder );
       s_pMetaObject->addProperty(
-         new PMProperty( "end1", &PMCylinder::setEnd1, &PMCylinder::end1 ) );
+         new PMCylinderProperty( "end1", &PMCylinder::setEnd1, &PMCylinder::end1 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "end2", &PMCylinder::setEnd2, &PMCylinder::end2 ) );
+         new PMCylinderProperty( "end2", &PMCylinder::setEnd2, &PMCylinder::end2 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "radius", &PMCylinder::setRadius, &PMCylinder::radius ) );
+         new PMCylinderProperty( "radius", &PMCylinder::setRadius, &PMCylinder::radius ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "open", &PMCylinder::setOpen, &PMCylinder::open ) );
+         new PMCylinderProperty( "open", &PMCylinder::setOpen, &PMCylinder::open ) );
    }
    return s_pMetaObject;
 }
@@ -201,7 +201,8 @@ void PMCylinder::restoreMemento( PMMemento* s )
 bool PMCylinder::isDefault( )
 {
    if( ( m_end1 == defaultEnd1 ) && ( m_end2 == defaultEnd2 )
-       && ( m_radius == defaultCylRadius ) && ( m_open == defaultOpen ) )
+       && ( m_radius == defaultCylRadius ) && ( m_open == defaultOpen )
+         && globalDetail( ) )
       return true;
    return false;
 }
@@ -214,41 +215,61 @@ void PMCylinder::createViewStructure( )
       m_pViewStructure->points( ).detach( );
    }
 
-   createPoints( m_pViewStructure->points( ), m_end1, m_end2, m_radius );
+   int steps = (int)( ( (float)s_numSteps / 2 ) * ( displayDetail( ) + 1 ) );
+   unsigned ptsSize = steps * 2;
+   unsigned lineSize = steps * 3;
+
+   if( ptsSize != m_pViewStructure->points( ).size( ) )
+      m_pViewStructure->points( ).resize( ptsSize );
+
+   createPoints( m_pViewStructure->points( ), m_end1, m_end2, m_radius, steps );
+
+   if( ( lineSize ) != m_pViewStructure->lines( ).size( ) )
+   {
+      m_pViewStructure->lines( ).detach( );
+      m_pViewStructure->lines( ).resize( lineSize );
+      createLines( m_pViewStructure->lines( ), steps );
+   }
 }
 
 PMViewStructure* PMCylinder::defaultViewStructure( ) const
 {
-   if( !s_pDefaultViewStructure )
+   if( !s_pDefaultViewStructure || s_pDefaultViewStructure->parameterKey( ) != viewStructureParameterKey( ) )
    {
-      s_pDefaultViewStructure = new PMViewStructure( s_numSteps * 2, s_numSteps * 3 );
-      PMLineArray & lines = s_pDefaultViewStructure->lines( );
+      delete s_pDefaultViewStructure;
+      s_pDefaultViewStructure = 0;
+      int steps = (int)( ( (float)s_numSteps / 2 ) * ( globalDetailLevel( ) + 1 ) );
+      s_pDefaultViewStructure = new PMViewStructure( steps * 2, steps * 3 );
 
       createPoints( s_pDefaultViewStructure->points( ), defaultEnd1,
-                    defaultEnd2, defaultCylRadius );
-
-      int i;
-      for( i = 0; i < ( s_numSteps - 1 ); i++ )
-      {
-          lines[i] = PMLine( i, i + 1 );
-          lines[i + s_numSteps] = PMLine( i + s_numSteps, i + s_numSteps + 1 );
-      }
-      lines[s_numSteps - 1] = PMLine( s_numSteps - 1, 0 );
-      lines[s_numSteps * 2 - 1] = PMLine( s_numSteps * 2 - 1, s_numSteps );
-
-      for( i = 0; i < s_numSteps; i++ )
-      {
-          lines[i + 2 * s_numSteps] = PMLine( i, i + s_numSteps );
-      }
+                    defaultEnd2, defaultCylRadius, steps );
 
+      createLines( s_pDefaultViewStructure->lines( ), steps );
    }
    return s_pDefaultViewStructure;
 }
 
-void PMCylinder::createPoints( PMPointArray& points, const PMVector& end1,
-                               const PMVector& end2, double radius )
+void PMCylinder::createLines( PMLineArray& lines, int steps )
 {
-   double angle = ( 2.0 * M_PI ) / ( double ) s_numSteps;
+   int i;
+   for( i = 0; i < ( steps - 1 ); i++ )
+   {
+         lines[i] = PMLine( i, i + 1 );
+         lines[i + steps] = PMLine( i + steps, i + steps + 1 );
+   }
+   lines[steps - 1] = PMLine( steps - 1, 0 );
+   lines[steps * 2 - 1] = PMLine( steps * 2 - 1, steps );
+
+   for( i = 0; i < steps; i++ )
+   {
+         lines[i + 2 * steps] = PMLine( i, i + steps );
+   }
+}
+
+void PMCylinder::createPoints( PMPointArray& points, const PMVector& end1,
+                               const PMVector& end2, double radius, int steps )
+{
+   double angle = ( 2.0 * M_PI ) / ( double ) steps;
 
    PMVector pointAt = end2 - end1;
    double pl = pointAt.abs( );
@@ -262,10 +283,10 @@ void PMCylinder::createPoints( PMPointArray& points, const PMVector& end1,
    endPoint *= radius;
 
    int i;
-   for( i = 0; i < s_numSteps; i++ )
+   for( i = 0; i < steps; i++ )
    {
       points[i] = PMPoint( endPoint + end1 );
-      points[i + s_numSteps] = PMPoint( endPoint + end2 );
+      points[i + steps] = PMPoint( endPoint + end2 );
       endPoint = rotation * endPoint;
    }
 }
