@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <stdio.h>
@@ -492,6 +492,7 @@ void KuickShow::showImage( const KFileItem *fi,
             m_viewer = new ImageWindow( kdata->idata, id, 0L, "image window" );
             s_viewers.append( m_viewer );
 
+	    connect( m_viewer, SIGNAL( nextSlideRequested() ), this, SLOT( nextSlide() ));
             connect( m_viewer, SIGNAL( destroyed() ), SLOT( viewerDeleted() ));
             connect( m_viewer, SIGNAL( sigFocusWindow( ImageWindow *) ),
                      this, SLOT( slotSetActiveViewer( ImageWindow * ) ));
@@ -501,6 +502,8 @@ void KuickShow::showImage( const KFileItem *fi,
                      this, SLOT( slotAdvanceImage( ImageWindow *, int )));
 	    connect( m_viewer, SIGNAL( pauseSlideShowSignal() ), 
 		     this, SLOT( pauseSlideShow() ) );
+            connect( m_viewer, SIGNAL (deleteImage ()),
+                     this, SLOT (slotDeleteImage ()));
             if ( s_viewers.count() == 1 && moveToTopLeft ) {
                 // we have to move to 0x0 before showing _and_
                 // after showing, otherwise we get some bogus geometry()
@@ -550,6 +553,21 @@ void KuickShow::showImage( const KFileItem *fi,
     } // isImage
 }
 
+void KuickShow::slotDeleteImage()
+{
+    KFileItemList list;
+    KFileItem *item = fileWidget->getCurrentItem(false);
+    list.append (item);
+    KFileItem *next = fileWidget->getNext(true);
+    if (!next)
+        next = fileWidget->getPrevious(true);
+        if (next)
+        showImage(next, false);
+    else
+        m_viewer->close(true);
+    fileWidget->del(list, false,false);
+}
+
 void KuickShow::startSlideShow()
 {
     KFileItem *item = kdata->slideshowStartAtFirst ?
@@ -561,14 +579,16 @@ void KuickShow::startSlideShow()
         fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( false );
         showImage( item, !oneWindowAction->isChecked(),
                    kdata->slideshowFullscreen );
-        m_slideTimer->start( kdata->slideDelay );
+	if(kdata->slideDelay)
+            m_slideTimer->start( kdata->slideDelay );
     }
 }
 
 void KuickShow::pauseSlideShow()
 {
     if(m_slideShowStopped) {
-	m_slideTimer->start( kdata->slideDelay );
+	if(kdata->slideDelay)
+	    m_slideTimer->start( kdata->slideDelay );
 	m_slideShowStopped = false;
     }
     else {
@@ -608,7 +628,8 @@ void KuickShow::nextSlide()
 void KuickShow::nextSlide( KFileItem *item )
 {
     m_viewer->showNextImage( item->url().path() );
-    m_slideTimer->start( kdata->slideDelay );
+    if(kdata->slideDelay)
+        m_slideTimer->start( kdata->slideDelay );
 }
 
 
@@ -718,7 +739,7 @@ void KuickShow::slotAdvanceImage( ImageWindow *view, int steps )
         QString filename;
         KIO::NetAccess::download(item->url(), filename, this);
         view->showNextImage( filename );
-        if (m_slideTimer->isActive())
+        if (m_slideTimer->isActive() && kdata->slideDelay)
             m_slideTimer->start( kdata->slideDelay );
 
         if ( kdata->preloadImage && item_next && item_next->url().isLocalFile() ) // preload next image
